%option noyywrap
%option caseless

%{
#include <stdio.h>
#include <cjson/cJSON.h>


#include "parser.tab.h"
extern YYSTYPE yylval;

#define cp_yylval_and_return(x) \
    yylval.sv = (char*)malloc(strlen(yytext) + 1); \
    strcpy(yylval.sv, yytext); \
    return x;
%}

%%

[ \t\n]	;
'[^'\n]*'	    {cp_yylval_and_return(STRING_V);}

[0-9]+\.[0-9]+ 	{yylval.fv = atof(yytext); return FLOAT_V;}
[0-9]+		    {yylval.iv = atoi(yytext); return INT_V;}


"CREATE"	{return CREATE;}
"INSERT"	{return INSERT;}
"INTO"		{return INTO;}
"SELECT"	{return SELECT;}
"FROM"		{return FROM;}
"WHERE"		{return WHERE;}
"VALUES"	{return VALUES;}
"DELETE"	{return DELETE;}
"UPDATE"	{return UPDATE;}
"SET"		{return SET;}
"JOIN"      {return JOIN;}
"TABLE"		{return TABLE;}
"DROP"		{return DROP;}

"INT"       {cp_yylval_and_return(INT_T);}
"FLOAT"     {cp_yylval_and_return(FLOAT_T);}
"STRING"    {cp_yylval_and_return(STRING_T);}
"AND"       return AND;
"OR"        return OR;
"NOT"       return NOT;
"AS"        return AS;
"IN"        return IN;
"ON"        return ON;

;		    {return NEWLINE;}

[*]			return *yytext;
[,]			return *yytext;
[.]         return *yytext;

[=]			{cp_yylval_and_return('=');}
[<]			{cp_yylval_and_return('<');}
[>]			{cp_yylval_and_return('>');}
[>][=]      {cp_yylval_and_return(">=");}
[<][=]      {cp_yylval_and_return("<=");}

[(]         return *yytext;
[)]         return *yytext;

"primary key"	{cp_yylval_and_return(PRIMARY_KEY);}

"quit"		{return QUIT;}

[a-zA-Z][a-zA-Z0-9]*	cp_yylval_and_return(IDENTIFIER);


%%
